/***************************************************************************
                          sposixthread.cpp  -  description
                             -------------------
    begin                : Sun Mar 30 2003
    copyright            : (C) 2003 by Andre Stein
    email                : andre_stein@web.de
    url                  : www.steinsoft.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sposixthread.h"

/** Global startfunction that calls sender's run
    function
 **/
void* posixThreadStartFunction(void* sender)
{
	//convert back to SPosixThread and execute
	//main run function
	((SPosixThread*)sender)->run();

	return 0;
}

SPosixThread::SPosixThread(void (*startFunction)())
: thread(0), threadStartFunction(startFunction),
  running(false)
{
}

SPosixThread::~SPosixThread()
{
}

/** No descriptions */
bool SPosixThread::start()
{
	//if already running, quit
	if (running)
		return true;	

	//Create Thread:
	//  - default attributes (2nd param)
	//  - global start function (3rd param)
	//  - pointer to this class which will be argument
	//     for global start function
	if (pthread_create(&thread, 0,
		posixThreadStartFunction,
		(void*) this))
	{
		return false;
	}

	//running..
	running = true;
	return true;
}

/** No descriptions */
void SPosixThread::stop(bool wait)
{
	//not running, nothing to stop, right?
	if (!running)
		return;
	/**
	  Stop Thread
	   - either wait until it has really stopped
	   - or send cancel request and resume other things
	 **/ 
	if (wait)
		pthread_join(thread, 0);
	else
		pthread_cancel(thread);

	//not running anymore
	running = false;
}

void SPosixThread::run()
{
	//if a start funciton has been set
	// : test is necessary to avoid seg fault
	if (threadStartFunction)
		threadStartFunction();
}

void SPosixThread::setRunFunction(void (*startFunction)())
{
	threadStartFunction = startFunction;
}
